@@ -34,17 +34,24 @@ class MissingLvmThinPool(Exception):
     pass
 
 
+class ThinPoolMetadataError(Exception):
+    pass
+
+
 class LVM(object):
     _lvs = LvmCLI.lvs
     _vgs = LvmCLI.vgs
     _lvcreate = LvmCLI.lvcreate
     _lvchange = LvmCLI.lvchange
     _lvremove = LvmCLI.lvremove
+    _lvextend = LvmCLI.lvextend
     _vgcreate = LvmCLI.vgcreate
     _vgchange = LvmCLI.vgchange
 
     @staticmethod
-    def _list_lv_full_names():
+    def _list_lv_full_names(filtr=""):
+        if filtr:
+            cmd = ["--noheadings", "-o", "lv_full_name", "--select", filtr]
         cmd = ["--noheadings", "-o", "lv_full_name"]
         raw = LVM._lvs(cmd)
         names = sorted(n.strip() for n in raw.splitlines())
@@ -52,8 +59,8 @@ class LVM(object):
         return names
 
     @classmethod
-    def list_lvs(cls):
-        lvs = [cls.LV.from_lvm_name(n) for n in cls._list_lv_full_names()]
+    def list_lvs(cls, filtr=""):
+        lvs = [cls.LV.from_lvm_name(n) for n in cls._list_lv_full_names(filtr)]
         log.debug("All LVS: %s" % lvs)
         return lvs
 
@@ -296,4 +303,25 @@ class LVM(object):
                            self.lvm_name])
             return vol
 
+        def _get_metadata_size(self):
+            args = ["--noheadings", "--nosuffix", "--units", "m",
+                    "-o", "metadata_percent,lv_metadata_size", self.lvm_name]
+            return map(float, LVM._lvs(args).split())
+
+        def _resize_metadata(self, size_mb):
+            args = ["--poolmetadatasize", "{}M".format(size_mb), self.lvm_name]
+            LVM._lvextend(args)
+
+        def check_metadata_size(self, resize=False):
+            min_size_mb = 1024
+            meta_pct, meta_sz = self._get_metadata_size()
+            log.debug("Pool: %s, metadata size=%sM (%s%%)" % (self.lvm_name,
+                                                              meta_sz,
+                                                              meta_pct))
+            if meta_sz < min_size_mb:
+                if resize:
+                    self._resize_metadata(min_size_mb)
+                else:
+                    raise ThinPoolMetadataError("Thinpool metadata too small")
+
 # vim: sw=4 et sts=4
