@@ -4,6 +4,7 @@ import os
 import shutil
 import glob
 import subprocess
+import json
 from ..utils import Rsync, File, BuildMetadata
 
 
@@ -16,9 +17,6 @@ def init(app):
 
 
 def add_argparse(app, parser, subparsers):
-    if not app.experimental:
-        return
-
     s = subparsers.add_parser("image-build",
                               help="Image build related tooling")
     s.add_argument("--postprocess", action="store_true",
@@ -26,6 +24,12 @@ def add_argparse(app, parser, subparsers):
     s.add_argument("--set-nvr",
                    help="Define the nvr of this build")
 
+    s = subparsers.add_parser("image-introspect",
+                              help="Informations around this image")
+    s.add_argument("--metadata",
+                   action="store_true",
+                   help="Print the metadata of this build (json)")
+
 
 def post_argparse(app, args):
     if args.command == "image-build":
@@ -33,6 +37,10 @@ def post_argparse(app, args):
             Postprocessor.postprocess(app)
         if args.set_nvr:
             BuildMetadata().set("nvr", args.set_nvr)
+    elif args.command == "image-introspect":
+        if args.metadata:
+            metadata = dict(BuildMetadata().items())
+            print(json.dumps(metadata, indent=2))
 
 
 def factorize(path):
@@ -102,9 +110,17 @@ def relocate_rpm_and_yum_dbs():
     # Make the /var entry a symlink to the moved db
     os.symlink("../../usr/share/rpm", "/var/lib/rpm")
 
-    log.info("Cleaning yum")
+    log.info("Relocating and cleaning yum")
+    # Delete everything under yum
     shutil.rmtree("/var/lib/yum")
 
+    # Then recreate it as a symlink
+    os.mkdir("/var/lib/yum")
+
+    shutil.move("/var/lib/yum", "/usr/share/yum")
+    # Make the /var entry a symlink to the moved path
+    os.symlink("../../usr/share/yum", "/var/lib/yum")
+
 
 @Postprocessor.add_step
 def disable_and_clean_yum_repos():
@@ -173,4 +189,12 @@ def clean_network_configs():
         os.unlink(fn)
 
 
+@Postprocessor.add_step
+def remove_iscsi_initiator_iqn():
+    """Remove the iSCSI initiator IQN, to ensure that none is set
+    A service is responsible for generating a new and uniqe name
+    FIXME https://bugzilla.redhat.com/show_bug.cgi?id=1393833
+    """
+    File("/etc/iscsi/initiatorname.iscsi").remove()
+
 # vim: sw=4 et sts=4
