@@ -83,8 +83,9 @@ def on_new_layer(imgbase, previous_lv, new_lv):
     try:
         # Some change in managed nodes is blapping /dev/mapper. Add it back
         # so LVM and /dev/mapper agree
-        LvmCLI.vgchange(["-ay"])
+        LvmCLI.vgchange(["-ay", "--select", "vg_tags = %s" % imgbase.vg_tag])
         remediate_etc(imgbase)
+        migrate_var(imgbase, new_lv)
         migrate_etc(imgbase, new_lv, previous_layer_lv)
     except:
         log.exception("Failed to migrate etc")
@@ -121,6 +122,25 @@ def on_post_init_layout(imgbase, existing_lv, new_base, new_layer):
     new_etc.mount()
 
 
+def migrate_var(imgbase, new_lv):
+    def strip(s):
+        return re.sub(r'^/tmp/mnt.*?/', '', s)
+
+    log.debug("Syncing items present in the new /var which are not "
+              "present in the existing FS")
+    with mounted(new_lv.path) as new_fs:
+        for cur, _dirs, files in os.walk(new_fs.path("/var")):
+            for d in _dirs:
+                newlv_path = "/".join([cur, d])
+                realpath = "/".join([strip(cur), d])
+                if not os.path.exists(realpath):
+                    log.debug("Copying {} to {}".format(newlv_path, realpath))
+                    if os.path.isdir(newlv_path):
+                        shutil.copytree(newlv_path, realpath, symlinks=True)
+                    else:
+                        shutil.copy2(newlv_path, realpath)
+
+
 def boot_partition_validation():
     """
     Function to validate all requirements for /boot partition
@@ -169,9 +189,10 @@ def remediate_etc(imgbase):
         s = re.sub(r'^/tmp/mnt.*?/', '', s)
         return re.sub(r'/+', '/', s)
 
-    def md5sum(a, b):
-        return hashlib.md5(open(a, 'rb').read()).hexdigest() == hashlib.md5(
-            open(b, 'rb').read()).hexdigest()
+    def sha256sum(a, b):
+        chksum_a = hashlib.sha256(open(a, 'rb').read()).hexdigest()
+        chksum_b = hashlib.sha256(open(b, 'rb').read()).hexdigest()
+        return chksum_a == chksum_b
 
     def diff_candidates(dc, problems, candidates=None):
         if candidates is None:
@@ -181,7 +202,7 @@ def remediate_etc(imgbase):
                 f = "{}/{}".format(dc.left, l)
                 if not os.path.islink(f):
                     if strip(f) in problems and strip(f) not in candidates:
-                        if md5sum(f, "{}/{}".format(dc.right, l)):
+                        if sha256sum(f, "{}/{}".format(dc.right, l)):
                             candidates.add(strip(f))
                             log.debug("Updating %s from the next "
                                       "layer" % ("{}".format(strip(f))))
@@ -240,7 +261,7 @@ def remediate_etc(imgbase):
         with mounted(imgbase._lvm_from_layer(layers[idx]).path) as m, \
                 mounted(imgbase._lvm_from_layer(layers[idx+1]).path) as n:
                     # Resync the files we changed on the last pass
-                    r = Rsync(checksum_only=True)
+                    r = Rsync(checksum_only=True, update_only=True)
                     r.sync(m.path("/etc"), n.path("/etc"))
 
                     check_layers(m, n)
@@ -289,6 +310,8 @@ def migrate_etc(imgbase, new_lv, previous_lv):
             log.error("New: %s" % new_rel)
 
         if is_same_product:
+            group_content, passwd_content = (None, None)
+
             # The IDMap check must be run before etc was copied!
             # The check relies on the fact that the old etc and new etc differ
             idmaps = IDMap(old_etc, new_fs.path("/usr/share/factory/etc"))
@@ -297,6 +320,8 @@ def migrate_etc(imgbase, new_lv, previous_lv):
                 log.debug("Drifted uids: %s gids: %s" %
                           idmaps.get_drift())
                 changes = idmaps.fix_drift(new_fs.path("/"))
+                group_content, passwd_content = idmaps.group_content, \
+                    idmaps.passwd_content
                 if changes:
                     log.info("UID/GID adjustments were applied")
                     log.debug("Changed files: %s" % list(changes))
@@ -328,6 +353,9 @@ def migrate_etc(imgbase, new_lv, previous_lv):
                     copy_files(new_fs.path("/") + c, [old_fs.path("/") + c],
                                "-a", "-r")
 
+            File(new_fs.path("/etc/group")).write(group_content)
+            File(new_fs.path("/etc/passwd")).write(passwd_content)
+
         else:
             log.info("Just copying important files")
             copy_files(new_etc,
@@ -344,7 +372,10 @@ def migrate_etc(imgbase, new_lv, previous_lv):
         fix_systemd_services(old_fs, new_fs)
         relocate_var_lib_yum(new_fs)
 
-        hack_rpm_permissions(new_fs)
+        with utils.bindmounted("/var", new_fs.path("/var")):
+            hack_rpm_permissions(new_fs)
+
+        relabel_selinux(new_fs)
 
         Motd(new_etc + "/motd").clear_motd()
 
@@ -386,6 +417,21 @@ def fix_systemd_services(old_fs, new_fs):
             log.exception("Could not remove %s. Is it a read-only layer?")
 
 
+def relabel_selinux(new_fs):
+    fc_path = new_fs.path("/")
+    fc_path += "/etc/selinux/targeted/contexts/files/file_contexts"
+
+    if not os.path.exists(fc_path):
+        log.debug("{} not found, not relabeling".format(fc_path))
+        return
+
+    dirs = ["/etc", "/var"]
+    setfiles = utils.ExternalBinary().setfiles
+
+    for d in dirs:
+        setfiles(["-r", new_fs.path("/"), fc_path, new_fs.path(d)])
+
+
 def relocate_var_lib_yum(new_fs):
     path = "/var/lib/yum"
     # Check whether /var is a symlink to /usr/share, and move it if it is not
